﻿using System;
using System.Collections.Generic;
using System.Threading;
using System.Threading.Tasks;

using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;

namespace Platform.Logging.ConsoleLogger
{
    [ProviderAlias("Console")]
    public class PlatformConsoleLoggerProvider : PlatformBatchLoggerProvider<PlatformConsoleLoggerOptions>
    {
        private static readonly ConsoleColors GrayedConsoleColors = new ConsoleColors(ConsoleColor.DarkGray, ConsoleColor.Black);

        public PlatformConsoleLoggerProvider( IConfigurationSection configuration )
            : base( configuration )
        {
        }

        protected override string SectionName => "Console";

        internal override PlatformBatchLogger<PlatformConsoleLoggerOptions> CreateLoggerImplementation(string categoryName)
        {
            return new PlatformConsoleLogger(this, categoryName);
        }

        protected override Task WriteMessagesAsync(IEnumerable<PlatformBatchLoggerMessage> messages, CancellationToken token)
        {
            bool showCategory = ProviderOptions.ShowCategoryInformation ?? true;

            foreach( var message in messages )
            {
                if (token.IsCancellationRequested)
                    return Task.CompletedTask;

                var level = GetLogLevelString(message.LogLevel);
                var colors = GetLogLevelConsoleColors(message.LogLevel);
                var header = string.Format(": {0:HH:mm:ss.fffff} {1}", message.Timestamp, message.Message );
                var info = string.Format("      {0:00000} [{1}] {2}", message.EventId.Id, message.EventId.Name, message.Category);

                Write(level, colors);
                WriteLine(header);

                if (showCategory)
                    WriteLine(info, GrayedConsoleColors);

                if (message.Exception != null)
                    WriteLine( "      " + message.Exception.ToString().Replace( "\r\n", "\r\n      " ), GrayedConsoleColors);
            }

            return Task.CompletedTask;
        }

        #region Tools

        private static void Write(string text, ConsoleColors colors = null)
        {
            if (colors != null && colors.Foreground.HasValue)
                System.Console.ForegroundColor = colors.Foreground.Value;
            if (colors != null && colors.Background.HasValue)
                System.Console.BackgroundColor = colors.Background.Value;

            System.Console.Out.Write(text);

            if (colors != null)
                if (colors.Foreground.HasValue || colors.Background.HasValue)
                    System.Console.ResetColor();
        }

        private static void WriteLine(string text, ConsoleColors colors = null)
        {
            if (colors != null && colors.Foreground.HasValue)
                System.Console.ForegroundColor = colors.Foreground.Value;
            if (colors != null && colors.Background.HasValue)
                System.Console.BackgroundColor = colors.Background.Value;

            System.Console.Out.WriteLine(text);

            if (colors != null)
                if (colors.Foreground.HasValue || colors.Background.HasValue)
                    System.Console.ResetColor();
        }

        private static ConsoleColors GetLogLevelConsoleColors(LogLevel logLevel)
        {
            switch (logLevel)
            {
                case LogLevel.Critical:
                    return new ConsoleColors(ConsoleColor.White, ConsoleColor.Red);
                case LogLevel.Error:
                    return new ConsoleColors(ConsoleColor.Black, ConsoleColor.Red);
                case LogLevel.Warning:
                    return new ConsoleColors(ConsoleColor.Yellow, ConsoleColor.Black);
                case LogLevel.Information:
                    return new ConsoleColors(ConsoleColor.DarkGreen, ConsoleColor.Black);
                case LogLevel.Debug:
                    return new ConsoleColors(ConsoleColor.Gray, ConsoleColor.Black);
                case LogLevel.Trace:
                    return new ConsoleColors(ConsoleColor.Gray, ConsoleColor.Black);
                default:
                    return null;
            }
        }

        #endregion
    }
}
